# 从Internet Archive（archive.org)下载指定item的meatdata
# 输入文件items_output.txt中包含要下载的item identifier，每行一个
# 输出文件在output_metadata'文件夹中，每个item一个.json文件
# 为了避免网络原因导致下载失败，设置了超时时间（timeout）10秒，失败后重试次数（retry_count）99次，可按需修改。
# 
# pip install requests
#
# 使用浏览器获取metadata：https://archive.org/metadata/20141029210045431
# 以下注释部分是使用internetarchive模块下载metadata的，但由于众所周知的网络问题，使用这个组件下载中出现网络错误不好处理，正式代码改为使用https直接下载元数据xml，
#from internetarchive import get_item
#ia_item = get_item('20141029210045431')
#metadata = ia_item.metadata.items()
#for k,v in metadata:
#    print(print(k,":",v))
# 
import requests
import os
import time

def download_metadata(id, retry_count=0):  
    url = f"https://archive.org/metadata/{id}"  
    try:  
        # 发送GET请求  
        response = requests.get(url, timeout=10)  # 设定一个合理的超时时间  
        response.raise_for_status()  # 如果请求失败，会抛出HTTPError异常  
  
        # 如果成功，将响应内容写入XML文件  
        output_dir = r'E:\mnbvc\warc\output_metadata'
        if not os.path.exists(output_dir):  
            os.makedirs(output_dir)
        json_file_path = os.path.join(output_dir, f"{id}.json")
        with open(json_file_path, "wb") as file:  
            file.write(response.content)  
        print(f"Downloaded metadata for {id} successfully.")  
    except (requests.exceptions.RequestException, FileNotFoundError) as e:  
        # 如果有任何请求相关的异常或文件写入错误，进行重试  
        if retry_count < 99:  # 假设最多重试5次  
            print(f"Failed to download metadata for {id} (retry {retry_count + 1}).")  
            time.sleep(1)  # 等待一段时间后再重试  
            download_metadata(id, retry_count + 1)  # 递归调用自身进行重试  
        else:  
            print(f"Exhausted retries for {id}. Giving up.")  
  
# 主程序  
if __name__ == "__main__":  
    with open("items_output.txt", "r") as file:  
        for line in file:  
            id = line.strip()  # 移除行尾的换行符（如果有的话）  
            download_metadata(id)