# 2024-5-16 ver2
# 从指定文件夹读取所有的Internet Archive（archive.org) item metadata (.json)文件，
# 将其中的.warc或.warc.gz文件生成下载链接
# 输入文件夹output_metadata，输出2个文件：
# 输出output_url\url_size.txt 按照warc文件大小降序排列，其中包含warc url和文件大小2个值，用逗号分隔
# 输出output_url\url.txt 只包含warc url，排序方式同样是按照warc文件大小降序排列
# 注意，由于此处可能同时存在warc和warc.gz，在没有下载并解压缩gz前，无法获得准确的文件大小，此处将压缩和不压缩的文件统一对待。
# 生成的url.txt导入下载工具可直接下载，IA不限制连接数和带宽
# 
# 已知问题：本程序没有考虑metadata中同时存在.warc和.warc.gz的情形，会重复生成2个下载链接。
# 已知问题：IA对于每个item，一般有2个服务器可提供服务，本程序只采用第一个服务器（d1），如有需要，将data.get('d1')改为data.get('d2')即可生成使用第二个服务器的下载链接

import os  
import json  
  
# 设置文件夹路径  
input_dir = 'E:\\mnbvc\\warc\\output_metadata'  
output_file_w_size = 'E:\\mnbvc\\warc\\output_url\\url_size.txt'
output_file = 'E:\\mnbvc\\warc\\output_url\\url.txt'
# 确保输出文件夹存在  
os.makedirs(os.path.dirname(output_file), exist_ok=True)  
  
# 创建一个列表来保存所有找到的URL  
urls = []  
  
# 遍历文件夹中的所有JSON文件  
for filename in os.listdir(input_dir):  
    if filename.endswith('.json'):  
        json_file_path = os.path.join(input_dir, filename)  
        try:  
            # 读取并解析JSON文件  
            with open(json_file_path, 'r', encoding='utf-8') as f:  
                data = json.load(f)  
            # 提取d1和dir属性的值  
            d1 = data.get('d1')  
            dir_value = data.get('dir')  
            # 检查是否提取到了d1和dir的值  
            if not d1 or not dir_value:  
                print(f"Missing d1 or dir in {filename}")  
                continue  
            # 假设files列表在data的一个名为'files'的键下（这取决于实际的JSON结构）  
            files = data.get('files', [])  # 如果JSON中没有'files'键，则默认为空列表  
            # 搜索以'.warc'或'warc.gz'结尾的name属性  
            found_warc = False  
            for file_info in files:  
                name = file_info.get('name')
                size = file_info.get('size')  # 获取warc文件大小
                if name and (name.endswith('.warc') or name.endswith('warc.gz')):  
                    found_warc = True  
                    url = 'https://' + d1 + '/' + dir_value + '/' + name  
                    urls.append({'url': url, 'size': size})
            # 如果没有找到".warc"或"warc.gz"文件，则打印错误信息  
            if not found_warc:  
                print(f"No .warc or warc.gz files found in {filename}")  
        except json.JSONDecodeError as e:  
            print(f"Error parsing JSON file {filename}: {e}")  
        except Exception as e:  
            print(f"Error processing file {filename}: {e}")  
  

#按照warc文件大小降序排序
urls.sort(key=lambda x: x['size'], reverse=True)
# 将所有URL、文件大小写入文件
with open(output_file_w_size, 'w', encoding='utf-8') as f:  
    for url in urls:  
        #f.write(url + '\n')
        f.write(f"{url['url']},{url['size']}\n")
# 只将所有URL写入文件
with open(output_file, 'w', encoding='utf-8') as f:  
    for url in urls:  
        #f.write(url + '\n')
        f.write(f"{url['url']}\n")